
from dataclasses import dataclass
from typing import Dict, List

@dataclass
class ChapterData:
    """Single chapter data class"""
    id: str
    chapter_name: str
    book_id: str

    def __init__(self, chapter_dict: Dict[str, str]):
        """Get chapter from dict"""
        self.id=chapter_dict['_id']
        self.chapter_name=chapter_dict['chapterName']
        self.book_id=chapter_dict['book']

@dataclass
class ChapterListData:
    """List of chapters data class"""
    chapters: List[ChapterData]
    total: int
    limit: int
    page: int
    pages: int
    has_more: bool


    def __init__(self, chapters_list_dict: Dict[str, str]):
        """Get chapters list from dict"""
        self.chapters=[ChapterData(chapter_dict) for chapter_dict in chapters_list_dict['docs']]
        self.total=int(chapters_list_dict['total'])
        self.limit=int(chapters_list_dict['limit'])
        self.page=int(chapters_list_dict['page'])
        self.pages=int(chapters_list_dict['pages'])
        self.has_more=int(chapters_list_dict['page']) < int(chapters_list_dict['pages'])




"""Filters to use with the OneApi SDK."""

from typing import List, Union
from theoneapi_sdk.enums import FilterType


class Filter:
    """Filter by specific fields."""

    def __init__(self, field: str, value: Union[str, List[str]], type: FilterType):
        """Initialize filter."""
        self.field = field
        self.value = value
        self.type = type


    def get_filter_string(self) -> str:
        """Get filter string."""
        if self.type == FilterType.MATCH:
            return f"{self.field}='{self.value}'"
        elif self.type == FilterType.NOT_MATCH:
            return f"{self.field}!='{self.value}'"
        elif self.type == FilterType.EXISTS:
            return f"{self.field}"
        elif self.type == FilterType.NOT_EXISTS:
            return f"!{self.field}"
        elif self.type == FilterType.INCLUDE:
            if isinstance(self.value, list):
                return f"{self.field}={','.join(self.value)}"
            else:
                return f"{self.field}={self.value}"
        elif self.type == FilterType.EXCLUDE:
            if isinstance(self.value, list):
                return f"{self.field}!={','.join(self.value)}"
            else:
                return f"{self.field}!={self.value}"
        elif self.type == FilterType.GREATER_THAN:
            return f"{self.field}>{self.value}"
        elif self.type == FilterType.LESS_THAN:
            return f"{self.field}<{self.value}"
        elif self.type == FilterType.GREATER_THAN_OR_EQUAL:
            return f"{self.field}>={self.value}"
        else:
            raise ValueError("Invalid filter type")




def get_filters_string(filters: List[Filter]) -> str:
    """Get filter string."""
    if len(filters) == 0:
        return ""
    else:
        return f"'&'{'&'.join([f.get_filter_string() for f in filters])}"



from dataclasses import dataclass
from typing import Dict, List

@dataclass
class CharacterData:
    """
    Character Data Class
    """
    id: str
    height: str
    race: str
    gender: str
    birth: str
    spouse: str
    death: str
    realm: str
    hair: str
    name: str
    wikiUrl: str

    def __init__(self, character_dict: Dict[str, str]):
        "Get Character from dict"
        self.id = character_dict['_id']
        self.height = character_dict['height']
        self.race = character_dict['race']
        self.gender = character_dict['gender']
        self.birth = character_dict['birth']
        self.spouse = character_dict['spouse']
        self.death = character_dict['death']
        self.realm = character_dict['realm']
        self.hair = character_dict['hair']
        self.name = character_dict['name']
        self.wikiUrl = character_dict['wikiUrl']

@dataclass
class CharacterDataList:
    """
    Character Data List Class
    """
    characters: List[CharacterData]
    total: int
    limit: int
    page: int
    pages: int
    has_more: bool

    def __init__(self, character_list_dict: Dict[str, str]):
        "Get Character List from dict"
        self.characters = [CharacterData(character_dict) for character_dict in character_list_dict['docs']]
        self.total = int(character_list_dict['total'])
        self.limit = int(character_list_dict['limit'])
        self.page = int(character_list_dict['page'])
        self.pages = int(character_list_dict['pages'])
        self.has_more=int(character_list_dict['page']) < int(character_list_dict['pages'])



from dataclasses import dataclass
from typing import Dict, List

@dataclass
class MovieData:
    """Single movie data class"""
    id: str
    runtime_in_minutes: int
    budget_in_millions: int
    box_office_in_millions: int
    academy_awards_wins: int
    academy_awards_nominated: int
    rotten_tomatoes_score: int

    def __init__(self, movie_dict: Dict[str, str]):
        """Get movie from dict"""
        self.id=movie_dict['_id']
        self.runtime_in_minutes=int(movie_dict['runtimeInMinutes'])
        self.budget_in_millions=int(movie_dict['budgetInMillions'])
        self.box_office_in_millions=int(movie_dict['boxOfficeRevenueInMillions'])
        self.academy_awards_wins=int(movie_dict['academyAwardWins'])
        self.academy_awards_nominated=int(movie_dict['academyAwardNominations'])
        self.rotten_tomatoes_score=int(movie_dict['rottenTomatoesScore'])

@dataclass
class MovieList:
    """List of movies data class"""
    movies: List[MovieData]
    total: int
    limit: int
    page: int
    pages: int
    has_more: bool


    def __init__(self, movies_list_dict: Dict[str, str]):
        """Get quotes list from dict"""
        self.movies=[MovieData(movie_dict) for movie_dict in movies_list_dict['docs']]
        self.total=int(movies_list_dict['total'])
        self.limit=int(movies_list_dict['limit'])
        self.page=int(movies_list_dict['page'])
        self.pages=int(movies_list_dict['pages'])
        self.has_more=int(movies_list_dict['page']) < int(movies_list_dict['pages'])



"""Handle the chapter of the API."""

from typing import TYPE_CHECKING, List
from theoneapi_sdk.chapter.chapter_dataclass import ChapterData, ChapterListData

if TYPE_CHECKING:
    from theoneapi_sdk.request_handler import RequestHandler
    from theoneapi_sdk.filter import Filter


class Chapter:
    """Handle the chapter endpoints."""

    def __init__(self, request_handler: 'RequestHandler'):
        """Initialize the class."""
        self._request_handler = request_handler

    def list(self, page: int = 1, limit: int = 100, filters: List["Filter"] = [], **kwargs) -> "ChapterListData":
        """Get a list of chapters.
        """
        return ChapterListData(self._request_handler._get(f"chapter?page={page}&limit={limit}", filters=filters, **kwargs))

    def chapter(self, chapter_id: str) -> ChapterData:
        """Get a chapter.
        """
        return ChapterData(self._request_handler._get(f"chapter/{chapter_id}")['docs'][0])



"""Handle the quote endpoint."""
from typing import TYPE_CHECKING, List
from theoneapi_sdk.quote.quote_dataclass import QuoteData, QuotesList

if TYPE_CHECKING:
    from theoneapi_sdk.request_handler import RequestHandler
    from theoneapi_sdk.filter import Filter


class Quote:
    """Handle the quote endpoint."""

    def __init__(self, request_handler: 'RequestHandler'):
        """Initialize the quote endpoint."""
        self._request_handler = request_handler

    def list(self, page: int = 1, limit: int = 100, filters: List["Filter"] = [], **kwargs) -> "QuotesList":
        """Get a list of quotes.
        """
        return QuotesList(self._request_handler._get(f"quote?page={page}&limit={limit}", filters=filters, **kwargs))

    def quote(self, quote_id: str) -> QuoteData:
        """Get a quote.
        """
        return QuoteData(self._request_handler._get(f"quote/{quote_id}")['docs'][0])




from dataclasses import dataclass
from typing import Dict, List

@dataclass
class QuoteData:
    """Single quote data class"""
    id: str
    dialog: str
    movie_id: str
    character_id: str

    def __init__(self, quote_dict: Dict[str, str]):
        """Get quote from dict"""
        self.id=quote_dict['_id']
        self.dialog=quote_dict['dialog']
        self.movie_id=quote_dict['movie']
        self.character_id=quote_dict['character']

@dataclass
class QuotesList:
    """List of quotes data class"""
    quotes: List[QuoteData]
    total: int
    limit: int
    page: int
    pages: int
    has_more: bool


    def __init__(self, quotes_list_dict: Dict[str, str]):
        """Get quotes list from dict"""
        self.quotes=[QuoteData(quote_dict) for quote_dict in quotes_list_dict['docs']]
        self.total=int(quotes_list_dict['total'])
        self.limit=int(quotes_list_dict['limit'])
        self.page=int(quotes_list_dict['page'])
        self.pages=int(quotes_list_dict['pages'])
        self.has_more=int(quotes_list_dict['page']) < int(quotes_list_dict['pages'])



"""Handle the movie endpoint."""
from typing import TYPE_CHECKING, List
from theoneapi_sdk.movie.movie_dataclass import MovieData, MovieList
from theoneapi_sdk.quote.quote_dataclass import QuotesList

if TYPE_CHECKING:
    from theoneapi_sdk.request_handler import RequestHandler
    from theoneapi_sdk.filter import Filter


class Movie:
    """Handle the movie endpoint."""

    def __init__(self, request_handler: "RequestHandler"):
        self._request_handler = request_handler

    def list(self, page: int = 1, limit: int = 100, filters: List["Filter"] = [], **kwargs) -> "MovieList":
        """Get a list of quotes.
        """
        return MovieList(self._request_handler._get(f"movie?page={page}&limit={limit}", filters=filters, **kwargs))
    
    def movie(self, movie_id: int) -> MovieData:
        """Get a movie by id.
        """
        return MovieData(self._request_handler._get(f"movie/{movie_id}")["docs"][0])

    def movie_quotes(self, movie_id: int, page: int = 1, limit: int = 100, filters: List["Filter"] = [], **kwargs) -> "QuotesList":
        """Get a list of quotes for a movie.
        """
        return QuotesList(self._request_handler._get(f"movie/{movie_id}/quote?page={page}&limit={limit}", filters=filters, **kwargs))



"""Test for the client module."""

import pytest
import theoneapi_sdk


def test_missing_token_client_init():
    """Test the client init."""
    with pytest.raises(ValueError) as excinfo:
        theoneapi_sdk.Client()
    assert str(excinfo.value) == "Token is required."



"""Handles all the books related requests."""

from typing import TYPE_CHECKING, Dict, List
from theoneapi_sdk.book.book_dataclass import BookData, BookList

if TYPE_CHECKING:
    from theoneapi_sdk.request_handler import RequestHandler
    from theoneapi_sdk.filter import Filter

class Book:
    """Handles all the books related requests."""

    def __init__(self, request_handler: 'RequestHandler'):
        """Initializes the Books class."""
        self._request_handler = request_handler

    def list(self, page: int = 1, limit: int = 100, filters: List["Filter"] = [], **kwargs) -> "BookList":
        """Gets a list of books."""
        return BookList(self._request_handler._get(f"/book?page={page}&limit={limit}", filters=filters, **kwargs))

    def book(self, book_id: str, **kwargs) -> BookData:
        """Gets a book by its ID."""
        return BookData(self._request_handler._get(f"/book/{book_id}", **kwargs)['docs'][0])

    def book_chapters(self, book_id: str, **kwargs) -> Dict[str, str]:
        """Gets a list of chapters for a book."""
        return self._request_handler._get('/book/{}/chapter'.format(book_id), **kwargs)



from dataclasses import dataclass
from typing import Dict, List

@dataclass
class BookData:
    id: str
    name: str

    def __init__(self, book_dict: Dict[str, str]):
        self.id = book_dict['_id']
        self.name = book_dict['name']


@dataclass
class BookList:
    """List of books"""
    books: List[BookData]
    total: int
    limit: int
    page: int
    pages: int
    has_more: bool

    def __init__(self, book_list_dict: Dict[str, str]):
        self.books = [BookData(book_dict) for book_dict in book_list_dict['docs']]
        self.total = int(book_list_dict['total'])
        self.limit = int(book_list_dict['limit'])
        self.page = int(book_list_dict['page'])
        self.pages = int(book_list_dict['pages'])
        self.has_more = book_list_dict['pages'] > book_list_dict['page']



"""Core client provides access to all the API endpoints."""

from theoneapi_sdk.request_handler import RequestHandler
from theoneapi_sdk.book.book import Book
from theoneapi_sdk.quote.quote import Quote
from theoneapi_sdk.movie.movie import Movie
from theoneapi_sdk.character.character import Character
from theoneapi_sdk.chapter.chapter import Chapter


_DEFUALT_URL: str = "https://the-one-api.dev/v2/"


class Client:
    """Provides access to all of "TheOne" API endpoints."""

    def __init__(self, token: str = None, api_url: str=_DEFUALT_URL):
        """Initialize the client."""

        if not token:
            raise ValueError("Token is required.")

        self.token = token
        self.api_url = api_url
        request_handler = RequestHandler(self.api_url, self.token)
        self._books_api = Book(request_handler)
        self._quotes_api = Quote(request_handler)
        self._movies_api = Movie(request_handler)
        self._characters_api = Character(request_handler)
        self._chapters_api = Chapter(request_handler)

    @property
    def book(self) -> "Book":
        """Provides access to the Books API."""
        return self._books_api


    @property
    def quote(self) -> "Quote":
        """Provides access to the Quotes API."""
        return self._quotes_api

    @property
    def movie(self) -> "Movie":
        """Provides access to the Movies API."""
        return self._movies_api

    @property
    def character(self) -> "Character":
        """Provides access to the Characters API."""
        return self._characters_api

    @property
    def chapter(self) -> "Chapter":
        """Provides access to the Chapters API."""
        return self._chapters_api



"""Handle the request to the OneAPI server."""

from typing import Dict, List, TYPE_CHECKING
import requests
from theoneapi_sdk.filter import get_filters_string

if TYPE_CHECKING:
    from theoneapi_sdk.filter import Filter

class RequestHandler:
    """Handle the request to the OneAPI server."""

    def __init__(self, api_url: str, token: str):
        """Initialize the RequestHandler object."""
        self.api_url = api_url
        self.token = token
        self.session = requests.Session()
        # TODO eliram: add support for timeout

    def _get_headers(self) -> Dict[str, str]:
        """Get the headers for the request."""
        headers = {
            "Authorization": f"Bearer {self.token}",
            "Content-Type": "application/json",
        }
        return headers

    def _request(self, method: str = "", endpoint: str = "", params: Dict[str,str] = {}, filters: List["Filter"] = [], data: Dict[str, str] = None) -> Dict[str, str]:
        """Make a request to the API."""

        if data is None:
            data = {}
        if not method:
            raise ValueError("Method is required.")
        if not endpoint:
            raise ValueError("Endpoint is required.")

        try:
            if method.lower() == "get":
                response = self.session.get(f"{self.api_url}{endpoint}{self._build_params(params)}{get_filters_string(filters)}", headers=self._get_headers(), params=data)
            elif method.lower() == "post":
                response = self.session.post(f"{self.api_url}{endpoint}", headers=self._get_headers(), json=data)
            elif method.lower() == "put":
                response = self.session.put(f"{self.api_url}{endpoint}", headers=self._get_headers(), json=data)
            elif method.lower() == "delete":
                response = self.session.delete(f"{self.api_url}{endpoint}", headers=self._get_headers(), json=data)
            else:
                raise ValueError("Method is not supported.")

            if response.status_code == 200:
                return response.json()
            else:
                raise ValueError(f"Request failed with status code {response.status_code}")
        #TODO eliram: move to own package error and exception
        except requests.exceptions.Timeout:
            raise ValueError("Request timed out.")
        except Exception as e:
            raise e

    def _get(self, endpoint: str = "", data: Dict[str, str] = {}, **kwargs ) -> Dict[str, str]:
        """Make a GET request to the API."""
        return self._request("get", endpoint, data, **kwargs)

    def _build_params(self, params: Dict[str, str] = {}) -> str:
        """Build the params for the request."""
        if not params:
            return ""
        return "?" + "&".join([f"{key}={value}" for key, value in params.items()])



"""Handle the charater endpoint."""
from typing import TYPE_CHECKING, List
from theoneapi_sdk.character.character_dataclass import CharacterData, CharacterDataList
from theoneapi_sdk.quote.quote_dataclass import QuotesList

if TYPE_CHECKING:
    from theoneapi_sdk.request_handler import RequestHandler
    from theoneapi_sdk.filter import Filter


class Character:
    """Handle the charater endpoint."""

    def __init__(self, request_handler: "RequestHandler"):
        self._request_handler = request_handler

    def list(self, page: int = 1, limit: int = 100, filters: List["Filter"] = [], **kwargs) -> "CharacterDataList":
        """Get a list of charaters.
        """
        return CharacterDataList(self._request_handler._get(f"character?page={page}&limit={limit}", filters=filters, **kwargs))
    
    def charater(self, charater_id: int) -> CharacterData:
        """Get a charater by id.
        """
        return CharacterData(self._request_handler._get(f"character/{charater_id}")["docs"][0])

    def charater_quotes(self, charater_id: int, page: int = 1, limit: int = 100, filters: List["Filter"] = [], **kwargs) -> "QuotesList":
        """Get a list of quotes for a charater.
        """
        return QuotesList(self._request_handler._get(f"character/{charater_id}/quote?page={page}&limit={limit}", filters=filters, **kwargs))




#!/usr/bin/env python
# This is a setup.py shim and NOTHING else should be added to it!!!
# we only need the setup.py file for local editable development. `pip install -e .`
import setuptools

if __name__ == "__main__":
    setuptools.setup()
# DO NOT ADD NOTHING TO THIS FILE. Everything should be in the pyproject.toml and setup.cfg



from enum import Enum


class FilterType(Enum):
    MATCH = "match"
    NOT_MATCH = "not_match"
    INCLUDE = "include"
    EXCLUDE = "exclude"
    EXISTS = "exists"
    NOT_EXISTS = "not_exists"
    REGEX = "regex"
    GREATER_THAN = "gt"
    LESS_THAN = "lt"
    GREATER_THAN_OR_EQUAL = "gte"
